@@ -18,6 +18,8 @@
 #define GPARTED_OPERATIONCHECK_H
 
 #include "../include/Operation.h"
+#include "../include/Partition.h"
+#include "../include/PartitionVector.h"
 
 namespace GParted
 {
@@ -26,10 +28,17 @@ class OperationCheck : public Operation
 {
 public:
 	OperationCheck( const Device & device, const Partition & partition ) ;
-	
-	void apply_to_visual( std::vector<Partition> & partitions ) ;
+	virtual ~OperationCheck();
+
+	void apply_to_visual( PartitionVector & partitions );
 
 private:
+	OperationCheck( const OperationCheck & src );              // Not implemented copy constructor
+	OperationCheck & operator=( const OperationCheck & rhs );  // Not implemented copy assignment operator
+
+	Partition & get_partition_new();
+	const Partition & get_partition_new() const;
+
 	void create_description() ;
 	bool merge_operations( const Operation & candidate );
 } ;
