@@ -1,4 +1,5 @@
-/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
+/* Copyright (C) 2004 Bart
+ * Copyright (C) 2008, 2009, 2010 Curtis Gedak
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -8,18 +9,18 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
- 
- 
-#ifndef DEFINE_JFS
-#define DEFINE_JFS
+
+
+#ifndef GPARTED_JFS_H
+#define GPARTED_JFS_H
 
 #include "../include/FileSystem.h"
+#include "../include/Partition.h"
 
 namespace GParted
 {
@@ -31,14 +32,13 @@ public:
 	void set_used_sectors( Partition & partition ) ;
 	void read_label( Partition & partition ) ;
 	bool write_label( const Partition & partition, OperationDetail & operationdetail ) ;
+	void read_uuid( Partition & partition ) ;
+	bool write_uuid( const Partition & partition, OperationDetail & operationdetail ) ;
 	bool create( const Partition & new_partition, OperationDetail & operationdetail ) ;
 	bool resize( const Partition & partition_new, OperationDetail & operationdetail, bool fill_partition = false ) ;
-	bool copy( const Glib::ustring & src_part_path, 
-		   const Glib::ustring & dest_part_path,
-		   OperationDetail & operationdetail ) ;
 	bool check_repair( const Partition & partition, OperationDetail & operationdetail ) ;
 };
 
 } //GParted
 
-#endif //JFS
+#endif /* GPARTED_JFS_H */
