@@ -16,14 +16,19 @@
  */
  
 #include "../include/ext2.h"
+#include "../include/OperationDetail.h"
+#include "../include/Partition.h"
+#include "../include/ProgressBar.h"
+#include "../include/Utils.h"
+
+#include <glibmm/ustring.h>
 
 namespace GParted
 {
 
 FS ext2::get_filesystem_support()
 {
-	FS fs ;
-	fs .filesystem = specific_type;
+	FS fs( specific_type );
 
 	fs .busy = FS::GPARTED ;
 
@@ -123,7 +128,7 @@ void ext2::set_used_sectors( Partition & partition )
 	//  unmounted.
 	if ( ! Utils::execute_command( dump_cmd + " -h " + partition .get_path(), output, error, true ) )
 	{
-		index = output .find( "Block count:" ) ;
+		Glib::ustring::size_type index = output.find( "Block count:" );
 		if ( index >= output .length() ||
 		     sscanf( output.substr( index ) .c_str(), "Block count: %Ld", &T ) != 1 )
 			T = -1 ;
@@ -146,7 +151,7 @@ void ext2::set_used_sectors( Partition & partition )
 			else
 			{
 				N = -1 ;
-				partition .messages .push_back( error ) ;
+				partition.push_back_message( error );
 			}
 		}
 		else
@@ -160,16 +165,19 @@ void ext2::set_used_sectors( Partition & partition )
 				N = Utils::round( N * ( S / double(partition .sector_size) ) ) ;
 		}
 
-		if ( T > -1 && N > -1 )
+		if ( T > -1 && N > -1 && S > -1 )
+		{
 			partition .set_sector_usage( T, N ) ;
+			partition.fs_block_size = S;
+		}
 	}
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 	
@@ -182,10 +190,10 @@ void ext2::read_label( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 		
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -205,10 +213,10 @@ void ext2::read_uuid( Partition & partition )
 	else
 	{
 		if ( ! output .empty() )
-			partition .messages .push_back( output ) ;
+			partition.push_back_message( output );
 
 		if ( ! error .empty() )
-			partition .messages .push_back( error ) ;
+			partition.push_back_message( error );
 	}
 }
 
@@ -220,12 +228,10 @@ bool ext2::write_uuid( const Partition & partition, OperationDetail & operationd
 
 bool ext2::create( const Partition & new_partition, OperationDetail & operationdetail )
 {
-	sigc::connection c = signal_progress.connect( sigc::mem_fun( *this, &ext2::create_progress ) );
-	bool ret = ! execute_command( mkfs_cmd + " -F -L \"" + new_partition.get_filesystem_label() + "\" " +
-	                              new_partition.get_path(),
-	                              operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
-	c.disconnect();
-	return ret;
+	return ! execute_command( mkfs_cmd + " -F -L \"" + new_partition.get_filesystem_label() + "\" " +
+	                          new_partition.get_path(),
+	                          operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE|EXEC_PROGRESS_STDOUT,
+	                          static_cast<StreamSlot>( sigc::mem_fun( *this, &ext2::create_progress ) ) );
 }
 
 bool ext2::resize( const Partition & partition_new, OperationDetail & operationdetail, bool fill_partition )
@@ -236,18 +242,15 @@ bool ext2::resize( const Partition & partition_new, OperationDetail & operationd
 		str_temp += " " + Utils::num_to_str( floor( Utils::sector_to_unit(
 					partition_new .get_sector_length(), partition_new .sector_size, UNIT_KIB ) ) ) + "K";
 
-	sigc::connection c = signal_progress.connect( sigc::mem_fun( *this, &ext2::resize_progress ) );
-	bool ret = ! execute_command( str_temp, operationdetail, EXEC_CHECK_STATUS );
-	c.disconnect();
-	return ret;
+	return ! execute_command( str_temp, operationdetail, EXEC_CHECK_STATUS|EXEC_PROGRESS_STDOUT,
+	                          static_cast<StreamSlot>( sigc::mem_fun( *this, &ext2::resize_progress ) ) );
 }
 
 bool ext2::check_repair( const Partition & partition, OperationDetail & operationdetail )
 {
-	sigc::connection c = signal_progress.connect( sigc::mem_fun( *this, &ext2::check_repair_progress ) );
 	exit_status = execute_command( fsck_cmd + " -f -y -v -C 0 " + partition.get_path(), operationdetail,
-	                               EXEC_CANCEL_SAFE );
-	c.disconnect();
+	                               EXEC_CANCEL_SAFE|EXEC_PROGRESS_STDOUT,
+	                               static_cast<StreamSlot>( sigc::mem_fun( *this, &ext2::check_repair_progress ) ) );
 	bool success = ( exit_status == 0 || exit_status == 1 || exit_status == 2 );
 	set_status( operationdetail, success );
 	return success;
@@ -257,91 +260,118 @@ bool ext2::move( const Partition & partition_new,
                  const Partition & partition_old,
                  OperationDetail & operationdetail )
 {
-	Sector distance;
-	Glib::ustring offset;
-	distance = partition_old.sector_start - partition_new.sector_start;
-	offset = Utils::num_to_str( llabs(distance) * partition_new.sector_size );
+	Sector distance = partition_old.sector_start - partition_new.sector_start;
+	Glib::ustring offset = Utils::num_to_str( llabs(distance) * partition_new.sector_size );
+	Glib::ustring cmd;
 	if ( distance < 0 )
-		return ! execute_command( image_cmd + " -ra -p -o " + offset + " " + partition_new.get_path(),
-		                          operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
+		cmd = image_cmd + " -ra -p -o " + offset + " " + partition_new.get_path();
 	else
-		return ! execute_command( image_cmd + " -ra -p -O " + offset + " " + partition_new.get_path(),
-		                          operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
+		cmd = image_cmd + " -ra -p -O " + offset + " " + partition_new.get_path();
 
+	fs_block_size = partition_old.fs_block_size;
+	return ! execute_command( cmd, operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE|EXEC_PROGRESS_STDERR,
+	                          static_cast<StreamSlot>( sigc::mem_fun( *this, &ext2::copy_progress ) ) );
 }
 
 bool ext2::copy( const Partition & src_part,
                  Partition & dest_part,
                  OperationDetail & operationdetail )
 {
+	fs_block_size = src_part.fs_block_size;
 	return ! execute_command( image_cmd + " -ra -p " + src_part.get_path() + " " + dest_part.get_path(),
-	                          operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE );
+	                          operationdetail, EXEC_CHECK_STATUS|EXEC_CANCEL_SAFE|EXEC_PROGRESS_STDERR,
+	                          static_cast<StreamSlot>( sigc::mem_fun( *this, &ext2::copy_progress ) ) );
 }
 
 //Private methods
 
 void ext2::resize_progress( OperationDetail *operationdetail )
 {
-	Glib::ustring ss;
-	size_t p = output.find_last_of('\n');
-	// looks like "Scanning inode table          XXXXXXXXXXXXXXXXXXXXXXXXXXXX------------"
-	if ( p == output.npos )
-		return;
-	ss = output.substr( p );
-	if ( ss.empty() )
-		return;
-	size_t sslen = ss.length();
-	if ( ss[sslen-1] != 'X' && ss[sslen-1] != '-' )
-		return;
-	// p = Start of progress bar
-	p = ss.find_last_not_of( "X-" );
-	if ( p == ss.npos )
-		p = 0;
-	else
-		p++;
-	size_t barlen = sslen - p;
-	// q = First dash in progress bar or end of string
-	size_t q = ss.find( '-', p );
-	if ( q == ss.npos )
-		q = sslen;
-	size_t xlen = q - p;
-	operationdetail->fraction = (double)xlen / barlen;
-	operationdetail->signal_update( *operationdetail );
+	Glib::ustring line = Utils::last_line( output );
+	size_t llen = line.length();
+	// There may be multiple text progress bars on subsequent last lines which look
+	// like: "Scanning inode table          XXXXXXXXXXXXXXXXXXXXXXXXXXXX------------"
+	if ( llen > 0 && ( line[llen-1] == 'X' || line[llen-1] == '-' ) )
+	{
+		// p = Start of progress bar
+		size_t p = line.find_last_not_of( "X-" );
+		if ( p == line.npos )
+			p = 0;
+		else
+			p++;
+		size_t barlen = llen - p;
+
+		// q = First dash in progress bar or end of string
+		size_t q = line.find( '-', p );
+		if ( q == line.npos )
+			q = llen;
+		size_t xlen = q - p;
+
+		operationdetail->run_progressbar( (double)xlen, (double)barlen );
+	}
+	// Ending summary line looks like:
+	// "The filesystem on /dev/sdb3 is now 256000 block long."
+	else if ( output.find( " is now " ) != output.npos )
+	{
+		operationdetail->stop_progressbar();
+	}
 }
 
 void ext2::create_progress( OperationDetail *operationdetail )
 {
-	Glib::ustring ss;
-	size_t p = output.find_last_of('\n');
-	// looks like "Writing inode tables: xx/yy"
-	if ( p == output.npos )
-		return;
-	ss = output.substr( p );
-	int x, y;
-	if ( sscanf( ss.c_str(), "\nWriting inode tables: %d/%d", &x, &y ) == 2 )
+	Glib::ustring line = Utils::last_line( output );
+	// Text progress on the LAST LINE looks like "Writing inode tables:  105/1600"
+	long long progress, target;
+	if ( sscanf( line.c_str(), "Writing inode tables: %Ld/%Ld", &progress, &target ) == 2 )
 	{
-		operationdetail->fraction = (double)x / y;
-		operationdetail->signal_update( *operationdetail );
+		operationdetail->run_progressbar( (double)progress, (double)target );
+	}
+	// Or when finished, on any line, ...
+	else if ( output.find( "Writing inode tables: done" ) != output.npos )
+	{
+		operationdetail->stop_progressbar();
 	}
 }
 
 void ext2::check_repair_progress( OperationDetail *operationdetail )
 {
-	Glib::ustring ss;
-	size_t p = output.find_last_of('\n');
-	// looks like "/dev/loop0p1: |==============================================  \ 95.1%"
-	if ( p == output.npos )
-		return;
-	ss = output.substr( p );
-	p = ss.find_last_of('%');
-	if ( p == ss.npos )
-		return;
-	ss = ss.substr( p-5, p );
-	float frac;
-	if ( sscanf( ss.c_str(), "%f%%", &frac ) == 1 )
+	Glib::ustring line = Utils::last_line( output );
+	// Text progress on the LAST LINE looks like
+	// "/dev/sdd3: |=====================================================   \ 95.1%   "
+	size_t p = line.rfind( "%" );
+	Glib::ustring pct;
+	if ( p != line.npos && p >= 5 )
+		pct = line.substr( p-5, 6 );
+	float progress;
+	if ( line.find( ": |" ) != line.npos && sscanf( pct.c_str(), "%f", &progress ) == 1 )
 	{
-		operationdetail->fraction = frac / 100;
-		operationdetail->signal_update( *operationdetail );
+		operationdetail->run_progressbar( progress, 100.0 );
+	}
+	// Only allow stopping the progress bar after seeing "non-contiguous" in the
+	// summary at the end to prevent the GUI progress bar flashing back to pulsing
+	// mode when the text progress bar is temporarily missing/incomplete before fsck
+	// output is fully updated when switching from one pass to the next.
+	else if ( output.find( "non-contiguous" ) != output.npos )
+	{
+		operationdetail->stop_progressbar();
+	}
+}
+
+void ext2::copy_progress( OperationDetail *operationdetail )
+{
+	Glib::ustring line = Utils::last_line( error );
+	// Text progress on the LAST LINE of STDERR looks like "Copying 146483 / 258033 blocks ..."
+	long long progress, target;
+	if ( sscanf( line.c_str(), "Copying %Ld / %Ld blocks", &progress, &target ) == 2 )
+	{
+		operationdetail->run_progressbar( (double)(progress * fs_block_size),
+		                                  (double)(target * fs_block_size),
+		                                  PROGRESSBAR_TEXT_COPY_BYTES );
+	}
+	// Or when finished, on any line of STDERR, looks like "Copied 258033 / 258033 blocks ..."
+	else if ( error.find( "\nCopied " ) != error.npos )
+	{
+		operationdetail->stop_progressbar();
 	}
 }
 
